/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.handler;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.core.Files;
import benten.core.Xliff;
import benten.core.XliffFileFilter;
import benten.twa.ui.AbstractWorkflowFileHandler;
import benten.twa.xliff.dialogs.ExtractXliffFragmentDialog;
import benten.ui.UiStatus;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTarget;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * XLIFF フラグメント抽出ハンドラーです。
 */
public class ExtractXliffFragmentHandler extends AbstractWorkflowFileHandler {

	public static class ExtractXliffFragmentStatus extends UiStatus {
		public File fromXliffFile;
		public File toDir;
		public String state;
		public int sizeLimit;
		public boolean includeDate;
	}

	private final String nowDate = new SimpleDateFormat("yyyyMMdd").format(new Date());

	@Override
	protected StatusDialog createDialog(Shell shell, IStructuredSelection selection) {
		setJobMessage("抽出中...");
		return new ExtractXliffFragmentDialog(shell, selection);
	}

	@Override
	protected void processFile(IStatus iStatus) throws IOException, SAXException {
		ExtractXliffFragmentStatus status = (ExtractXliffFragmentStatus) iStatus;
		Xliff inXliff = Xliff.loadInstance(status.fromXliffFile);
		int outXliffIndex = 0;
		Xliff outXliff = Xliff.newInstance(createFile(status, ++outXliffIndex));

		for (BlancoXliffFile inFile : inXliff.getFileList()) {
			BlancoXliffFile outFile = createBlancoXliffFile(outXliff, inFile);

			for (BlancoXliffTransUnit inUnit : inFile.getBody().getTransUnitList()) {
				if (status.state != null) {
					BlancoXliffTarget target = inUnit.getTarget();
					String state = (target == null) ? "" : target.getState();
					state = (state == null) ? "" : state;
					if (!status.state.equals(state)) {
						continue;
					}
				}
				outFile.getBody().getTransUnitList().add(inUnit);
				if (status.sizeLimit > 0 && outXliff.getAllTransUnitList().size() == status.sizeLimit) {
					outXliff.save();
					outXliff = Xliff.newInstance(createFile(status, ++outXliffIndex));
					outFile = createBlancoXliffFile(outXliff, inFile);
				}
			}
		}
		outXliff.save();
	}

	protected File createFile(ExtractXliffFragmentStatus status, int fileIndex) {
		StringBuilder name = new StringBuilder();
		name.append(Files.baseName(status.fromXliffFile));
		if (status.sizeLimit > 0) {
			name.append("-" + fileIndex);
		}
		if (status.includeDate) {
			name.append("-" + nowDate);
		}
		return XliffFileFilter.appendExtension(new File(status.toDir, name.toString()));
	}

	protected BlancoXliffFile createBlancoXliffFile(Xliff outXliff, BlancoXliffFile inFile) {
		BlancoXliffFile outFile = outXliff.createBlancoXliffFile();
		outFile.setSourceLanguage(inFile.getSourceLanguage());
		outFile.setTargetLanguage(inFile.getTargetLanguage());
		outFile.setDatatype(inFile.getDatatype());
		outFile.setHeader(inFile.getHeader());
		return outFile;
	}
}
