/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.handler;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.core.Xliff;
import benten.core.XliffFileFilter;
import benten.twa.ui.AbstractWorkflowDirectoryHandler;
import benten.twa.xliff.dialogs.MergeXliffDialog;
import benten.ui.UiStatus;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * XLIFF フラグメント・マージ・ハンドラーです。
 */
public class MergeXliffHandler extends AbstractWorkflowDirectoryHandler {

	public static class MergeXliffStatus extends UiStatus {
		public File fromFile;
		public File toDir;
	}

	@Override
	protected StatusDialog createDialog(Shell shell, IStructuredSelection selection) {
		setJobMessage("マージ中...");
		return new MergeXliffDialog(shell, selection);
	}

	@Override
	public void run(IStatus iStatus) {
		MergeXliffStatus status = (MergeXliffStatus) iStatus;
		explore(status, status.toDir, XliffFileFilter.INSTANCE);
	}

	@Override
	protected void processFile(IStatus iStatus, File toXliffFile) throws IOException, SAXException {
		MergeXliffStatus status = (MergeXliffStatus) iStatus;
		Xliff inXliff = Xliff.loadInstance(status.fromFile);
		Xliff toXliff = Xliff.loadInstance(toXliffFile);

		for (BlancoXliffTransUnit inUnit : inXliff.getAllTransUnitList()) {
			BlancoXliffTransUnit toUnit = toXliff.getTransUnit(inUnit.getId());
			toUnit.setTranslate(inUnit.getTranslate());
			toUnit.setContextGroupList(inUnit.getContextGroupList());
			toUnit.setTarget(inUnit.getTarget());
			toUnit.setNoteList(inUnit.getNoteList());
		}
		toXliff.save();
	}
}
